
* Exercises, Beginner Group
* ATAI-IITA Stata Training
* 25 January, 2012

clear
cap log close
set mem 50m
set more off

* Note: The goal of these exercises is to familiarize ourselves with some of the basic tools learned thus far.
*		There are usually multiple correct ways to do the same task in Stata.

*****************************************************************************************************************************
*****************************************************************************************************************************

*EXERCISE 1: USING DATA, GENERATING VARIABLES, REPORTING DESCRIPTIVE STATS

// 0) Prepping the dataset and log file

*	i) Set your current directory
*cd ...

*	ii) Load the mumuadu dataset.
use mumuadu_dataset,clear

*	iii) Open a log file called exercise1 to record the output from this exercise.
log using exercise1, text replace


// 1) Constructing variables with multiple criteria.

*	i) Construct a variable identifying males with a cognitive score of 4.
gen maleCog4 = female==0 & cogscore==4

*	ii) Construct a variable identifying males with a cognitive score of 2 from Bunso village.
gen maleCog2Bunso = female==0 & cogscore==5 & village=="Bunso"

*	iii) Construct a variable identifying males with a cognitive score of 5 from Bunso village over the age of 30.
gen maleCog5BunsoOver30 = female==0 & cogscore==5 & village=="Bunso" & age>=30

*	iv) For each of the above variables, determine what share of the sample satisfy the given criteria (hint: tabulate).
tab maleCog4
tab maleCog2Bunso
tab maleCog5BunsoOver30


// 2) Using -egen- and other commands.

*	i) Construct a variable identifying the mean landholding size within each village.
egen landAvg_byVillage = mean(landsize), by(village)

*	ii) Construct a variable that takes the log difference between farmer i's landholding size and his/her village mean.
gen landLogMeanDev = log(landsize/landAvg_byVillage)

*	iii) Construct a variable identifying the maximum age in each village.
egen maxAge_byVillage = max(age), by(village)

*	iv) Construct a variable identifying the standard deviation of revenue for each level of amiguity.
egen sdRev_byAmbiguity = sd(revenuetot), by(ambiguity)


// 3) Compare variables across villages.
*NOTE: You can of course do the below exercises village by village using "if" statements, but try to calculate the stats for all villages in one step.

*	i) Determine the share of individuals in each village that took a loan.
tab village loan, row

*	ii) Determine the median revenue in each village. 
table village, c(median revenuetot)

*	iii) Show in a single table the mean, median and standard deviation of revenue, age, and cognitive score.
tabstat revenuetot age cogscore, stat(mean median sd)

*	iv) Repeat (iii) with the calculation of each statistic within village. 
tabstat revenuetot age cogscore, stat(mean median sd) by(village)


*--->STOP: RETURN TO LECTURE SLIDES


*****************************************************************************************************************************
*****************************************************************************************************************************

// EXERCISE 2: GRAPHING AND STATISTICAL ANALYSIS

// 1) Graphing. For each of the below, produce the graph and save it for viewing later.

*	i) Plot the distribution of total revenue.
histogram reveneutot, bin(20)
kdensity revenuetot
graph save revenueDistribution, replace

*	ii) Plot the relationship between total revenue and land size. Is there a linear relationship? (hint: scatter)
scatter revenuetot landsize
graph save revenueLandScatter, replace

*	iii) Plot the distribution of total revenue below the 95th percentile (hint: summarize, detail).
sum revenuetot, detail
histogram revenuetot if landsize<=r(p95), bin(20)
graph save revenueDistributionP95, replace

*	iv) Plot the distribution of total revenue between the 1st and 99th percentile (hint: summarize, detail).
sum revenuetot, detail
histogram revenuetot if landsize>=r(p1) & landsize<=r(p99), bin(20) // also try: if inrange(landsize,r(p1),r(p99))
graph save revenueDistributionP1P99, replace

*	v) Compare the kernel density of landsize in Bunso and Akim Odumese on the same graph using a bandwidth of 0.5 for each.
*		(hint: twoway (kdensity ... if...) (kdensity ... if ...), legend(...))
twoway (kdensity landsize if village=="Bunso", bw(0.5)) (kdensity landsize if village=="Akim Odumese", bw(0.5)), legend(label(1 "Bunso") label(2 "Akim Odumese"))

*Open some of the graphs and experiment with some of the graph editing tools.


// 2) Basic statistical inference
*NOTE: In all regression frameworks, use clustered standard errors as in our example.

*	i) Do farmers taking loans produce more maize and garden egg crops. Put a p-value on your statement.
*		(use quantmaizec and quantgeggc)
ttest quantmaizec, by(loan)
ttest quantgeggc, by(loan)

*	ii) Show how to recover the information from a t-test (ttest) using a regression (regress).
ttest quantmaizec, by(loan)
reg quantmaizec loan
display "the sum of loan+_cons " _b[loan]+_b[_cons] "is equivalent to the mean revenue for those taking loans"
display "the coefficient " _b[_cons] "on the constant is equivalent the mean revenue for those not taking loans"
display "the coefficient " _b[loan] "on loan is equivalent to the abs(diff) line in the t-test"

*	iii) Conditional on age, do female farmers report lower revenue than men? Put a p-value on your statement.
*		(hint: conditional ---> regression framework)
reg revenuetot female age, cluster(cluster)

*	iv) Conditional on age and landsize, are there revenue differentials across reported levels of ambiguity?
*		(hint: i.ambiguity)
reg revenuetot age landsize i.ambiguity, cluster(cluster)

*	v) Using existing variables in the dataset, build and estimate a reasonable (and parsimonious) regression to explain the prevelance of health insurance.
*		(you may want to use probit instead of regress)


*--->STOP: RETURN TO LECTURE SLIDES


*****************************************************************************************************************************
*****************************************************************************************************************************

// EXERCISE 3: POWER CALCULATION

* Written by Angela Ambroz (Research Manager, J-PAL Global). 

* In this exercise, we're going to take you through some commands for
* determining a sufficient sample size - where by "sufficient", we mean
* a sample size that ensures both statistical significance AND enough power.

* First, a reminder:
**		POWER is the likelihood that, when a program has an effect, you'll be able to detect it
**		(i.e. distinguish it from zero).
**		SIGNIFICANCE is the likelihood that the measured effect didn't occur by chance. 

* As you saw from the lecture, the "industry standard" for minimum power is 80%. But the
* higher, the better!

* As you ALSO saw from the previous portion of this exercise, power can be calculated
* quite easily using the Optimal Design software. In fact, there are a number of ways
* to calculate power, and they should all give you the same results:
**	- 	Optimal Design: simple, visual.
**	-	Stata: replicable, better for tweaking.
**	- 	Excel: great for understanding the fundamentals.

use power_data.dta, clear
* This is the dataset we'll be using. This is anonymized baseline data from a current J-PAL project in
* Rwanda. The three variables we have here are:
**	hhid 	= Household ID
**	villid 	= Village ID
**	cherry 	= The quantity of coffee cherry produced by that household. This will be our measure of productivity,
**			  in other words, our outcome variable! We'll be conducting our power calculation on this. 

**	ESTIMATING PARAMETERS	**

loneway cherry villid
* A large one-way analysis of variance (ANOVA) is a speedy way to get the intra-class correlation coefficient
* - that is, rho. In this case, we're measuring how much variance in household's coffee cherry production
* can be attributed to the village they're in.


global rho = r(rho)
* Here, we use the "global" command to define a new macro. This macro will call up the intra-class correlation
* coefficient (which Stata remembers right now as "r(rho)") whenever we need it. In plain language, we're saving
* rho so we can use it later. 

sum cherry
global control = r(mean)
global sd = r(sd)
* Now we do some summary stats on "cherry", and save them for later. Specifically, we're saving the 
* mean of cherry, and its standard deviation.

* The treatment effect:
global effect = .10
global treatment = (1+$effect)*$control
* We set the treatment effect as 10% of the baseline mean of cherry. In other words, by the endline,
* we expect the treated group to have an average that is 110% of the control. 


**	POWER CALCULATION	**

* EXERCISE 3.1: Limiting the number of households/villages to 50. 

sampsi $control $treatment, p(.8) sd1($sd)
* You always have to use sampsi before using sampclus. If this was NOT a clustered randomized trial,
* we would only use sampsi. Note that sampsi's default alpha value is 5%, and its default power level
* is 90%. Here, in the options, we're setting power to 80% ("p(.8)") and the baseline standard deviation
* to the values we obtained from the above ANOVA ("sd1($sd)"). ("$sd" calls up the macro.)

sampclus, obsclus(50) rho($rho)
* Now we use sampclus to obtain the minimum number of villages needed for that power level. We
* set the number of households per village to 50 ("obsclus(50)").

sampsi $control $treatment, p(.9) sd1($sd)
sampclus, obsclus(50) rho($rho)
* Here, we do the same for a 90% power level. Compare these values to what Optimal Design gives you.

* EXERCISE 3.2: Limiting the number of villages to 150.

sampsi $control $treatment, p(.8) sd1($sd)
sampclus, numclus(150) rho($rho)
* As you can see, to limit the number of households per village/cluser, we use the "obsclus" option.
* To limit the number of clusters themselves, we use "numclus". Here, we test for 80%.

sampsi $control $treatment, p(.9) sd1($sd)
sampclus, numclus(150) rho($rho)
* Using the commands you learned, try to write some lines of code that will give us the 
* sample size needed for 90% power given a limit of 150 villages. How many households would
* you need to sample? How do these results compare with what Optimal Design gave you?


*--->STOP: RETURN TO LECTURE SLIDES



*****************************************************************************************************************************
*****************************************************************************************************************************

// EXERCISE 4: RANDOMIZATION

* WE HAVE A NUMBER OF SCHOOLS THAT WE WANT TO ASSIGN TO EITHER TREATMENT OR CONTROL.
* IN OUR FIRST EXAMPLE, WE WILL DO A SIMPLE RANDOMIZATION
* FOR THE SECOND EXAMPLE, WE WILL STRATIFY BY THE SCHOOL'S GENDER COMPOSITION 
* AND LANGUAGE OF INSTRUCTION
* THIRD EXAMPLE, WE WILL STRATIFY BY LANGUAGE, GENDER AND 
*	AVERAGE PRE-TEST SCORE OF THE SCHOOL (A CONTINUOUS VARIABLE)

* THIS DATA SET (RandomizationExercise_balsakhi_data.dta) HAS 5 IMPORTANT VARIABLES
*	schoolid 
*	language
*	gender
*	pretest_mean
* (IT ALSO HAS AREA ID, BUT THAT CAN BE IGNORED)

use "RandomizationExercise_balsakhi_data.dta", clear

// EXERCISE 4.1: SIMPLE RANDOMIZATION 

* WE CREATE A RANDOM NUMBER
*** BEFORE CREATING A RANDOM NUMBER, SORT BY A UNIQUE ID, THEN SET A SEED
*** SO YOU CAN RECREATE YOUR SAME RANDOM NUMBER SEQUENCE EVERY TIME
sort schoolid
set seed 12345
gen random = uniform()

* SORT BY THIS RANDOM NUMBER:
* (BECAUSE THE SORTING IS RANDOM, THERE IS NO STATISTICAL REASON WHY 
* THE FIRST HALF OF THE OBSERVATIONS WOULD BE ANY DIFFERENT FROM THE SECOND HALF)
sort random

* WE CREATE A VARIABLE "treatment" WHICH EQUALS 1 IF TREATMENT AND 0 IF CONTROL
gen treatment = 0
replace treatment = 1 if _n <= _N/2
* OR: gen treatment =_n <= _N/2
* WE COULD HAVE DECIDED THAT treatment = 1 WHEN _n > _N

* WE NOW HAVE A VARIABLE THAT DEFINES WHICH SCHOOLS ARE TREATMENT 
* AND WHICH ARE CONTROL

***********************
* RESET
drop random treatment
***********************

// EXERCISE 4.2: STRATIFY BY LANGUAGE AND GENDER *

* CREATE A RANDOM NUMBER
sort schoolid
set seed 12345
gen random = uniform()

* WITHIN EACH LANGUAGE GENDER, SORT RANDOMLY
sort language gender random

* FIGURE OUT HOW MANY SCHOOLS IN EACH STRATUM
by language gender: gen strata_size = _N

* ASSIGN A VALUE REFLECTING THE CURRENT (RANDOM) ORDER OF THESE SCHOOLS IN EACH STRATUM
by language gender: gen strata_index = _n

* CREATE A VARIABLE "treatment" WHICH EQUALS 1 IF TREATMENT AND 0 IF CONTROL
* WHICH IS BASED ONLY ON THE RANDOM ORDER
gen treatment = 0
replace treatment = 1 if strata_index <= (strata_size/2)

* WE NOW HAVE A VARIABLE THAT DEFINES WHICH SCHOOLS ARE TREATMENT 
* AND WHICH ARE CONTROL, STRATIFIED BY LANGUAGE AND GENDER

************************************************
* RESET
drop random treatment strata_size strata_index 
************************************************

// EXERCISE 4.3: STRATIFY BY language AND pretest_mean RATE *

* CREATE A RANDOM NUMBER
sort schoolid
set seed 12345
gen random = uniform()

* SORT BY LANGUAGE GENDER AND pretest_mean
sort language gender pretest_mean

* FIGURE OUT HOW MANY SCHOOLS IN EACH STRATUM
by language gender: gen strata_size = _N

* WE WANT TO SPLIT SCHOOLS INTO "GROUPS" of 2
* WHERE EACH GROUP REPRESENTS ONE TREATMENT SCHOOL AND ONE CONTROL, AND HAS SIMILAR pretest_mean
* SO, FOR EXAMPLE, IF THERE ARE 30 SCHOOLS IN A STRATUM, 
* WE WANT TO BREAK THE STRATUM INTO 15 GROUPS: (30/2 = 15) 
by language gender: gen group = group(strata_size/2)

* WITHIN EACH GROUP, SORT RANDOMLY
sort language group random

* FIGURE OUT HOW MANY SCHOOLS IN EACH GROUP 
*	(SHOULD BE 2, BUT MAY BE 1 IF THERE ARE AN ODD NUMBER OF SCHOOLS)
*	(IT IS WORTH NOTING THAT USING THIS CODE, THE SCHOOL WITH THE 
*	HIGHEST pretest_mean IN EACH "ODD" language WILL ALWAYS BE THE ONE WITH NO PAIR) 
sort language gender group
by language gender group: gen groupsize = _N


* ASSIGN A VALUE REFLECTING THE CURRENT (RANDOM) ORDER OF THESE SCHOOLS IN EACH GROUP
* (SHOULD TAKE ON A VALUE OF 1 OR 2)
by language gender group: gen groupindex = _n
* IF THE GROUP HAS NO PAIR, MAKE THE VALUE == 0
replace groupindex = 0 if groupsize == 1

* WE CREATE A VARIABLE "treatment" WHICH EQUALS 1 IF TREATMENT AND 0 IF CONTROL
* (FOR THE SCHOOLS THAT ARE IN GROUPS OF 2)
gen treatment = 0
replace treatment = 1 if groupindex == 1

* FOR THE SINGLE SCHOOLS (WITH NO PAIR IN A GROUP), WE CAN RANDOMIZE AS IF THEY 
* ARE ALL EQUIVALENT AND PART OF THE SAME LANGUAGE/GENDER
sum school if groupindex == 0
scalar oddSCHOOL = r(N)

sort groupindex random
replace treatment = 1 if _n <= (oddSCHOOL/2)

* WE NOW HAVE A VARIABLE THAT DEFINES WHICH SCHOOLS ARE TREATMENT 
* AND WHICH ARE CONTROL, STRATIFIED BY language GENDER AND pretest_mean

sort language gender pretest_mean

* Close your log file.
log close

*-->STOP: RETURN TO LECTURE SLIDES
